/*****************************************************************************
 *            Created: 6/04/2011
 *          Copyright: (C) Copyright David Goodwin, 2011
 *            License: GNU General Public License
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#include "commentedit.h"

#include <QPlainTextEdit>
#include <QTextStream>
#include <QTextBlock>

CommentEdit::CommentEdit(QWidget *parent) :
    QPlainTextEdit(parent)
{
}

void CommentEdit::keyPressEvent(QKeyEvent *e) {

    const int totalMaxLen = 6144;  // six lines of 1024 characters
    const int lineMax = 6;
    const int lineMaxLen = 1024;

    // First check that we aren't trying to pass the maximum character count
    // for the field
    qint64 textLength = toPlainText().length();

    int key = e->key();

    if (!e->text().isEmpty() &&
        key != Qt::Key_Backspace &&
        key != Qt::Key_Delete) {
        // The user is inserting text.

        /*
         * Enforce total character count restrictions.
         */
        if (textLength > totalMaxLen)  {
            e->accept();
            return;
        }

        /*
         * Enforce line count restrictions.
         */
        int lineCount = blockCount();
        if (key == Qt::Key_Enter || key == Qt::Key_Return) {
            if (lineCount >= lineMax) {
                e->accept();
                return;
            }
        }

        /*
         * Enforce line length restrictions.
         */
        if ((textCursor().block().text().length() >= lineMaxLen) &&
            !(key == Qt::Key_Enter || key == Qt::Key_Return)) {
            e->accept();
            return;
        }
    }

    QPlainTextEdit::keyPressEvent(e);
}
