/*
-----------------------------------------------------
   Application Name: "DIMG"
Application Version: 1.1.0
          File Name: imagefileutils.h
         Start Date: 27/02/2010
       Last Changed: $LastChangedDate: 2011-04-09 00:17:11 +1200 (Sat, 09 Apr 2011) $
           Revision: $Revision: 295 $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
   File Description: ImageFileUtils Class header
-----------------------------------------------------
        This file is part of "DIMG".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

        File Notes:
        ------------------
*/


#ifndef IMAGEFILEUTILS_H
#define IMAGEFILEUTILS_H

#include <QObject>
#include <QSettings>
#include <QPointer>
#include <QFileDialog>
#include <QtDebug>
#include <QFileInfo>
#include <QFile>
#include <QProgressDialog>

#include <imagefile.h>

#include "imagefilefactory.h"
#include "errors.h"

/** This class provides a set of utility functions for handling IamgeFiles.
 */
class ImageFileUtils : public QObject
{
Q_OBJECT
public:
    /** Class Constructor. Does nothing much.
     */
    ImageFileUtils(QObject *parent = 0) : QObject(parent) {}

    /** Set the QSettings object to be used to retrieve settings.
      Must be called before initialise().

      \param settings The QSettings instance to get settings from.
     */
    void setSettings(QSettings * settings);

    /** Initialises class for use. You must call setSettings() first.
     */
    void initialise();

    /** Displays a file open dialog and returns the image file the user
      selected (or null if the user canceled).
      \param imageFile [out] The image file the user opened.
      \returns An error code.
     */
    int openImage(ImageFile ** imageFile);

    /** Attempts to open the requested file and returns the result.
     \param imageFile The opened ImageFile
     \param filename the file to open
     \returns An error code.
     */
    int openImage(ImageFile **imageFile, QString filename);

    /** Creates a new disk image of the requested size.
     \param ImageFile the opened image file
     \param The expected size the image will be
     \returns An error code.
     */
    int createImage(ImageFile ** imageFile, long long expectedSize);

    /** Prompts the user to save oldImageFile as newImageFile. If the
      user canceles newImageFile will be set to null.

      If the user does not cancel then newImageFile will be created,
      all data from oldImageFile will be copied into it and then
      ImageFile::save() will be called on it. The newly created image
      file will then be returned through newImageFile. oldImageFile will
      remain valid.

      \param oldImageFile The image file to save
      \param newImageFile The new image file the user has just saved (output)
      \returns An error code.
     */
    int saveImageAs(ImageFile ** oldImageFile, ImageFile **newImageFile);
private:
    QString getSaveAsFilename(QString filename);

    QPointer<QSettings> settings;
    ImageFileFactory factory;
};

#endif // IMAGEFILEUTILS_H
