/*
-----------------------------------------------------
   Application Name: "DIMG"
          File Name: main.cpp
         Start Date: 27/06/2008
       Last Changed: $LastChangedDate: 2011-04-09 00:17:11 +1200 (Sat, 09 Apr 2011) $
           Revision: $Revision: 295 $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2008,2009
            License: GNU General Public License
   File Description: Program entry point
-----------------------------------------------------
	This file is part of "DIMG".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

	File Notes:
	------------------
*/


// ------------------
//   Header Files for Qt:
// ------------------

// Qt stuff
#include <QApplication>
#include <QtDebug>
#include <QStringList>
#include <QFile>
#include "Dimg.h"

/** Program entry point.
 */
int main(int argc, char *argv[])
{
    /*
    Steps that this function performs:
          0) Setup program
          1) Load settings
          2) Create app object (runs constructor)
          3) Process command line args
          4) Create dimg object (runs constructor)
          5) Load file if one has been specified
          6) Pass control to the app object
    */
   	
    // Step 0: Setup program
    QCoreApplication::setOrganizationName("zxnet");
    QCoreApplication::setOrganizationDomain("zx.net.nz");
    QCoreApplication::setApplicationName("DIMG");

    // Step 1: Load settings
    QScopedPointer<QSettings> settings;
    QString settingsFile;
    settingsFile = QCoreApplication::applicationName() + ".ini";
    qDebug() << "Checking for configuration file " << settingsFile;

    if (QFile::exists(settingsFile)) {
        // Load settings from there
        settings.reset(new QSettings(settingsFile, QSettings::IniFormat, 0));
    } else {
        settings.reset(new QSettings());
    }
    qDebug() << "Settings Loaded.";

    QString styleName = settings->value("UI/Style","PlatformDefault").toString().toUpper();
    QString styleSheet = settings->value("UI/StyleSheetFile","").toString();

    // Note how PLATFORMDEFAULT is excluded. We'll let Qt decide what to do if that
    // comes up.
    if (styleName == "MOTIF" ||
        styleName == "CDE" ||
        styleName == "WINDOWS" ||
        styleName == "PLASTIQUE" ||
        styleName == "CLEANLOOKS") {
        QApplication::setStyle(styleName);
    }

    // Step 2:
    QApplication app(argc, argv);
    if (!styleSheet.isEmpty()) {
        QFile ss(styleSheet);
        if (ss.open(QFile::ReadOnly)) {
            app.setStyleSheet(ss.readAll());
            ss.close();
        }
    }

    // Step 3: Process command line args
    QString filename;
    QStringList args = app.arguments();
    if (args.count() > 1) {
        // We have been given arguments to interpret.
        // We only support one at the moment - open file.
        filename = args.at(1);
    }

    // Step 4:		(pass in apps memory address so it can be used by dimg)
    Dimg dmg(&app);

    // Step 5: Open a file if we've been told to do so
    if (!filename.isEmpty())
        dmg.loadFile(filename);

    // Step 6: Main Event Loop
    return app.exec();
}
