/*
-----------------------------------------------------
   Application Name: DIMG - FSContainer
          File Name: fileproperties.cpp
         Start Date: unknown - ask subversion.
       Last Changed: $LastChangedDate: 2011-04-09 00:17:11 +1200 (Sat, 09 Apr 2011) $
           Revision: $Revision: 295 $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2008,2009,2010
            License: GNU General Public License
   File Description: File Properties dialog implementation
-----------------------------------------------------
        This file is part of DIMG.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

File Notes:
------------------
*/


#include "fileproperties.h"
#include "ui_fileproperties.h"

FileProperties::FileProperties(QWidget *parent) :
    QDialog(parent),
    m_ui(new Ui::FileProperties)
{
    m_ui->setupUi(this);
}

FileProperties::~FileProperties()
{
    delete m_ui;
}

void FileProperties::setFileEntry(FSEntry *e) {
    m_ui->lblIcon->setPixmap(e->getIcon().pixmap(32,32));
    m_ui->leFileName->setText(e->fileName());
    m_ui->leLocation->setText(QDir::toNativeSeparators("/" + e->path()));
    m_ui->leType->setText(e->suffix() + " File");

    m_ui->lblSize->setText(QString::number(e->getSize()));
    m_ui->lblDOSFilename->setText(e->fileName());

    // Attributes
    FSPlugin::Attributes a = e->getAttributes();
    m_ui->chkArchive->setChecked(a.testFlag(FSPlugin::A_Archive));
    m_ui->chkHidden->setChecked(a.testFlag(FSPlugin::A_Hidden));
    m_ui->chkReadOnly->setChecked(a.testFlag(FSPlugin::A_ReadOnly));
    m_ui->chkSystem->setChecked(a.testFlag(FSPlugin::A_System));

    // Date created
    if (e->getDate(FSEntry::D_CreatedDate).isValid()) {
        m_ui->dteCreated->setDate(e->getDate(FSEntry::D_CreatedDate));
        m_ui->dteCreated->setTime(e->getTime(FSEntry::D_CreatedTime));
    }
    else {
        m_ui->dteCreated->setVisible(false);
        m_ui->lblDateCreated->setVisible(false);
    }

    // Date modified
    if (e->getDate(FSEntry::D_ModifiedDate).isValid()) {
        m_ui->dteModified->setDate(e->getDate(FSEntry::D_ModifiedDate));
        m_ui->dteModified->setTime(e->getTime(FSEntry::D_ModifiedTime));
    } else {
        m_ui->dteModified->setVisible(false);
        m_ui->lblDateModified->setVisible(false);
    }

    // Date accessed
    if (e->getDate(FSEntry::D_AccessedDate).isValid())
        m_ui->deAccessed->setDate(e->getDate(FSEntry::D_AccessedDate));
    else {
        m_ui->deAccessed->setVisible(false);
        m_ui->lblDateAccessed->setVisible(false);
    }
}

void FileProperties::changeEvent(QEvent *e)
{
    switch (e->type()) {
    case QEvent::LanguageChange:
        m_ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
