/*
-----------------------------------------------------
   Application Name: "DIMG"
         Start Date: 09/07/2010
       Last Changed: $LastChangedDate: $
           Revision: $Revision: $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
-----------------------------------------------------
        This file is part of "DIMG".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

        File Notes:
        ------------------
*/

#ifndef ABSTRACTFAT_H
#define ABSTRACTFAT_H

#include <QObject>
#include <QList>
#include <QIODevice>
#include <QPointer>

#include "biosparamblock.h"
#include "errors.h"

namespace FilesystemFAT {
    /** This is the base class for all File Allocation Table types (12bit,
        16bit and 32bit).

        The getChain() and getValue() functions must be implemented by any
        subclasses.

        The default constructor just stores the supplied QIODevice and
        BiosParamBlock in the dev and bpb private member variables.
     */
    class AbstractFAT : public QObject
    {
    Q_OBJECT
    public:
        /** Constructs a File Allocation Table interface object.
          \param device The QIODevice to load the FAT from
          \param bpb The BIOS Parameter Block
          \param parent The parent QObject
         */
        explicit AbstractFAT(QIODevice *device,
                             BiosParamBlock *bpb,
                             QObject *parent = 0);

        /** Loads the chain of clusters for the file starting with the specified
            start cluster.

            \param startCluster The first cluster in the file
            \returns A list of all the clusters in the file
         */
        virtual QList<unsigned int> getChain(unsigned int startCluster) = 0;

        /** Gets the value for the specified cluster.

          \param clusterNumber the cluster number to get the value for.
          \returns The value for the specified cluster.
         */
        virtual unsigned int getValue(unsigned int clusterNumber) = 0;

        /** Returns the last error code to occur.
          \returns The last error code to occur.
         */
        ReturnCode lastError() {return lastErr; }

    protected:
        /** The QIODevice used for accessing the FAT
         */
        QPointer<QIODevice> dev;

        /** The BIOS Parameter Block used for locating the FAT
         */
        QPointer<BiosParamBlock> bpb;

        /** The last error to occur
         */
        ReturnCode lastErr;
    };
}
#endif // ABSTRACTFAT_H
