/*
-----------------------------------------------------
   Application Name: "DIMG"
         Start Date: 10/07/2010
       Last Changed: $LastChangedDate: $
           Revision: $Revision: $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
-----------------------------------------------------
        This file is part of "DIMG".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

        File Notes:
        ------------------
*/

#include "dataarea.h"
#include "constants.h"

#include <QtDebug>

using namespace FilesystemFAT;

DataArea::DataArea(QIODevice *dev,
                   BiosParamBlock *bpb,
                   QObject *parent) :
    QObject(parent)
{
    qDebug() << "Data Area init...";
    this->dev = dev;
    this->bpb = bpb;

    Q_ASSERT_X(bpb != 0,
               "DataArea Constructor",
               "BIOS Parameter Block is NULL");
    Q_ASSERT_X(dev != 0,
               "DataArea Constructor",
               "QIODevice is NULL");

    /* Ok. We should compute the space the data area occupies. First up - the
     * start sector. The first bit of the offset is:
     *      Number of Reserved Sectors
     *    + Number of FATs * Size of a FAT
     * which is then multiplied by the number of bytes in a sector.
     */
    startSector = bpb->reservedSectors();
    startSector += bpb->numberOfFATs() * bpb->sectorsPerFAT();
    startSector *= bpb->bytesPerSector();

    /* Now we just have to add on the space allocated to the root directory
       table.
     */
    startSector += bpb->maxRootEntries() * Constants::bytesPerDirEntry;

    qDebug() << "Data Area Start Sector:" << startSector;

    lastErr = Errors::SUCCESS;
}

QByteArray DataArea::readCluster(int cluster) {

    if (dev.isNull() || bpb.isNull()) {
        lastErr = Errors::BAD_PTR;
        return QByteArray();
    }

    const unsigned int dataStart = startSector +
                                   ((cluster - 2) * bpb->sectorsPerCluster())
                                   * bpb->bytesPerSector();

    const int clusterDataSize = bpb->sectorsPerCluster() *
                                bpb->bytesPerSector();

    QByteArray clusterData;

    if (!dev->seek(dataStart)) {
        lastErr = Errors::SEEK_ERR;
        return QByteArray();
    }

    /* Pull the cluster data from the QIODevice */
    clusterData = dev->read(clusterDataSize);

    lastErr = Errors::SUCCESS;
    return clusterData;
}
