/*
-----------------------------------------------------
   Application Name: "DIMG"
         Start Date: 10/07/2010
       Last Changed: $LastChangedDate: $
           Revision: $Revision: $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
-----------------------------------------------------
        This file is part of "DIMG".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

        File Notes:
        ------------------
*/

#ifndef DATAAREA_H
#define DATAAREA_H

#include <QObject>
#include <QIODevice>
#include <QPointer>
#include <QByteArray>

#include "biosparamblock.h"
#include "errors.h"

namespace FilesystemFAT {

    /** This represents the filesystems data area. It is used for retrieving
        clusters by cluster number. It is used by DataChain along with an
        AbstractFAT to access file data chains.
     */
    class DataArea : public QObject
    {
    Q_OBJECT
    public:
        /** Constructs a DataArea object. It computes the start and end
            addresses based on the data in the BIOS Parameter Block.
          \param dev The QIODevice used for reading clusters.
          \param bpb The BIOS Parameter Block.
          \param parent The parent QObject.
         */
        explicit DataArea(QIODevice *dev,
                          BiosParamBlock *bpb,
                          QObject *parent = 0);

        /** Reads the specified cluster and returns its data.
          \param cluster The cluster to read.
          \returns The clusters data.
         */
        QByteArray readCluster(int cluster);

        /** Returns the last error to occur. This should be checked after using
          functions such as readCluster().
          \returns The last error to occur.
         */
        ReturnCode lastError() {return lastErr; }

    private:       
        QPointer<QIODevice> dev;    /*!< The QIODevice for reading clusters */
        QPointer<BiosParamBlock> bpb; /*!< The BIOS Parameter Block */
        unsigned int startSector;   /*!< The first sector */
        ReturnCode lastErr;         /*!< The last error to occur */
    };
}

#endif // DATAAREA_H
