/*
-----------------------------------------------------
   Application Name: "DIMG"
         Start Date: 09/07/2010
       Last Changed: $LastChangedDate: $
           Revision: $Revision: $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
-----------------------------------------------------
        This file is part of "DIMG".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

        File Notes:
        ------------------
*/

#ifndef FAT12_H
#define FAT12_H

#include "abstractfat.h"

#include <QByteArray>

namespace FilesystemFAT {
    /** Handles the 12-bit File Allocation Table used by FAT12.
     */
    class Fat12 : public AbstractFAT
    {
    Q_OBJECT
    public:
        /** Constructs a FAT12 file allocation table object to access
          the specified FAT.
          \param device The QIODevice to access the FAT from
          \param bpb The BiosParameterBlock to locate the FAT with
          \param fatNumber the File Allocation Table to use
          \param parent The parent QObject.
         */
        explicit Fat12(QIODevice *device,
                       BiosParamBlock *bpb,
                       unsigned int fatNumber,
                       QObject *parent = 0);

        /** Constructs a FAT12 file allocation table object to access
          the first FAT.
          \param device The QIODevice to access the FAT from
          \param bpb The BiosParameterBLock to locate the FAT with
          \param parent The parent QObject.
        */
        explicit Fat12(QIODevice *device,
                       BiosParamBlock *bpb,
                       QObject *parent = 0);

        virtual QList<unsigned int> getChain(unsigned int startCluster);

        virtual unsigned int getValue(unsigned int clusterNumber);
    private:
        QByteArray fatData;     /*!< The current FAT is stored in here */
        unsigned int fatNumber; /*!< The FAT we are working with */

        void init();
    };
}

#endif // FAT12_H
