/*
-----------------------------------------------------
   Application Name: "DIMG"
         Start Date: 09/07/2010
       Last Changed: $LastChangedDate: $
           Revision: $Revision: $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
-----------------------------------------------------
        This file is part of "DIMG".

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

        File Notes:
        ------------------
*/

#include "fsfile.h"
#include "fatfs.h"
#include "directory.h"

using namespace FilesystemFAT;

FsFile::FsFile(QByteArray entryData,
               FatFS* filesystem,
               QString parentPathName,
               QObject *parent) :
    DirectoryEntry(entryData,parent)
{
    this->filesystem = filesystem;
    this->parentPathName = parentPathName;

    // Create a new instance of the DataChain we will be using.
    dc.reset(new DataChain(filesystem,
                           this->entryCluster(),
                           this->size(),
                           this));

    // Copy the last error code from the DC. If it succeeded, we succeeded. If
    // it failed, we failed.
    lastErr = dc->lastError();
}

FsFile::FsFile(QByteArray entryData,
               FatFS *filesystem,
               QString parentPathName,
               QString longFilename,
               QObject *parent):
DirectoryEntry(entryData,parent)
{
    this->filesystem = filesystem;
    this->parentPathName = parentPathName;
    this->mLongFilename = longFilename;

    // Create a new instance of the DataChain we will be using.
    dc.reset(new DataChain(filesystem,
                           this->entryCluster(),
                           this->size(),
                           this));

    // Copy the last error code from the DC. If it succeeded, we succeeded. If
    // it failed, we failed.
    lastErr = dc->lastError();
}

Directory *FsFile::getParentDirectory() {
    Directory *dir = filesystem->createDirectory(parentPathName);
    if (filesystem->lastError() != Errors::SUCCESS) {
        lastErr = filesystem->lastError();
        return NULL;
    } else {
        lastErr = Errors::SUCCESS;
        return dir;
    }
}


/* ****************************************************************** *
 * These functions all just pass through to the DataChain and then    *
 * copy the error code.                                               *
 * ****************************************************************** */

bool FsFile::seek(unsigned int pos){
    bool val = dc->seek(pos);
    lastErr = dc->lastError();
    return val;
}

unsigned int FsFile::pos() {
    unsigned int val = dc->pos();
    lastErr = dc->lastError();
    return val;
}

unsigned int FsFile::length() {
    unsigned int val = dc->length();
    lastErr = dc->lastError();
    return val;
}

QByteArray FsFile::read(unsigned int length) {
    QByteArray val = dc->read(length);
    lastErr = dc->lastError();
    return val;
}

QByteArray FsFile::readAll() {
    QByteArray val = dc->readAll();
    lastErr = dc->lastError();
    return val;
}

bool FsFile::atEnd() {
    bool val = dc->atEnd();
    lastErr = dc->lastError();
    return val;
}
