#ifndef _LIBDSKIMAGEFILE_H
#define _LIBDSKIMAGEFILE_H

#include <imagefile.h>
#include "../../DQlibdskdrv/qlibdsk.h"
#include <QFile>
#include <QBuffer>
#include <QMessageBox>
#include <QSettings>
#include <QScopedPointer>
#include <QProgressDialog>
#include <QtDebug>
#include <QCoreApplication>

/** A file format plugin wrapper around LibDsk
 */
class LibDskImageFile : public ImageFile
{
Q_OBJECT
public:
    explicit LibDskImageFile(QObject *parent = 0);
    ~LibDskImageFile();

    /** Opens an image file, optionally creating it if it does not
      already exist.
     \param filename The file to open or create
     \param create If the file should be created if it does not exist
     \param cache If the file should be cached in memory or not. This setting
        is ignored if the file size is greater than the maximum cache size.
     \param expectedSize the expected size of the image. This is used when
        creating images only.
     */
    void open(QString filename,
              bool create,
              bool cache,
              long long expectedSize);

    /** Opens an existing image file. It can optionally cache the entire image
      file in memory if desired.

      \param filename The file to open.
      \param cache If the file should be cached in memory provided it is not too large.
     */
    void openExisting(QString filename, bool cache);

    /** Gets the name of the currently open image file.
      \returns The name of the currently open image file.
     */
    QString fileName() const { return filename; }

    /** A device which can be used for read/write access to the image file
      \returns A QIODevice which can be used to read from and write to the device. This
      pointer belongs to the RawImageFile and must not be deleted.
     */
    QIODevice *device();

    /** Saves the image file if any changes have been made.
      \returns An error code.
     */
    int save();

    /** Closes the image without saving it. Call save() to save it first.
      \returns An error code.
     */
    int close();

    /** Gets the features supported by this image file.
      \param The image files supported features.
      */
    ImageFile::Features getFeatures() const;

    /** Gets the comment
      \returns The images comment
     */
    QString getComment() const;

    /** Sets the comment
      \param comment The new comment.
     */
    void setComment(QString comment);

    /** Is the comment open?
      \returns the answer
     */
    bool isOpen() const;

    /** Returns the last error encountered by LibDsk
     */
    int getLastError() const {return lastError;}
    QString getErrorString(int errorCode) {return QLibDsk::strerror(errorCode);}
signals:

private slots:
    void bytesWritten(qint64 /*bytes*/);

private:
    bool createCachedFile();
    void cleanFile();
    void dirtyFile();

    QString filename;

    QBuffer *buffer;

    bool dirty;

    // LibDsk access stuff
    QLibDsk drive;
    int lastError;
    bool driveOpen;
    DSK_GEOMETRY diskGeom;
};

#endif // _LIBDSKIMAGEFILE_H
