/*
-----------------------------------------------------
   Application Name: DIMG
          File Name: rawimagefile.h
         Start Date: 27/02/2010
       Last Changed: $LastChangedDate: 2011-04-09 00:17:11 +1200 (Sat, 09 Apr 2011) $
           Revision: $Revision: 295 $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
   File Description: RawImageFile header
-----------------------------------------------------
        This file is part of DIMG.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

File Notes:
------------------
*/


#ifndef RAWIMAGEFILE_H
#define RAWIMAGEFILE_H

#include <imagefile.h>
#include <QFile>
#include <QBuffer>
#include <QMessageBox>
#include <QSettings>
#include <QScopedPointer>
#include <QProgressDialog>
#include <QtDebug>
#include <QCoreApplication>

class RawImageFile : public ImageFile
{
Q_OBJECT
public:
    explicit RawImageFile(QObject *parent = 0);
    ~RawImageFile();

    /** Opens a raw image file, optionally creating it if it does not
      already exist.
     \param filename The file to open or create
     \param create If the file should be created if it does not exist
     \param cache If the file should be cached in memory or not. This setting
        is ignored if the file size is greater than the maximum cache size.
     \param expectedSize the expected size of the image. This is used when
        creating images only.
     */
    void open(QString filename,
              bool create,
              bool cache,
              long long expectedSize);

    /** Opens an existing image file. It can optionally cache the entire image
      file in memory if desired.

      \param filename The file to open.
      \param cache If the file should be cached in memory provided it is not too large.
     */
    void openExisting(QString filename, bool cache);

    /** Gets the name of the currently open image file.
      \returns The name of the currently open image file.
     */
    QString fileName() const { return filename; }

    /** A device which can be used for read/write access to the image file
      \returns A QIODevice which can be used to read from and write to the device. This
      pointer belongs to the RawImageFile and must not be deleted.
     */
    QIODevice *device();

    /** Saves the image file if any changes have been made.
      \returns An error code.
     */
    int save();

    /** Closes the image without saving it. Call save() to save it first.
      \returns An error code.
     */
    int close();

    /** Gets the features supported by this image file.
      \param The image files supported features.
      */
    ImageFile::Features getFeatures() const;

    // The raw format plugin doesn't support comments.
    QString getComment() const {return "";}
    void setComment(QString /*comment*/) {}
    long maxCommentLength() {return 0;}

    bool isOpen() const;

    bool isDirty() const {return dirty;}
signals:

private slots:
    void bytesWritten(qint64 /*bytes*/);

private:
    bool createCachedFile();
    void cleanFile();
    void dirtyFile();

    QString filename;

    QFile *file;
    QBuffer *buffer;
    QIODevice *currentDevice;

    bool cached;
    bool createdInCache;
    bool dirty;
};

#endif // RAWIMAGEFILE_H
