/*
-----------------------------------------------------
   Application Name: DIMG
          File Name: fpzip.h
         Start Date: 28/03/2010
       Last Changed: $LastChangedDate: 2011-04-09 00:17:11 +1200 (Sat, 09 Apr 2011) $
           Revision: $Revision: 295 $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
   File Description: FPzip header
-----------------------------------------------------
        This file is part of DIMG.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

File Notes:
------------------
*/


#ifndef _FPZIP_H
#define _FPZIP_H

// Qt Headers
#include <QString>
#include <QStringList>
#include <QFile>
#include <QFileInfo>
#include <QtPlugin>
#include <QtDebug>
#include <QFileInfo>

// Plugin Headers
#include <imagefileplugin.h>
#include <imagefile.h>

#include "versinfo.h"
#include "zipimagefile.h"

/** Raw Image File Format plugin
 */
class FPzip : public QObject, public ImageFilePlugin {

    Q_OBJECT
    Q_INTERFACES(ImageFilePlugin)

public:

    FPzip() {}
    ~FPzip() {}

    bool fileIsSupported(QString filename);

    QStringList getNameFilters();

    ImageFile *createImageFile(QString filename,
                               ImageFilePlugin::LoadHints hints,
                               long long expectedSize);

    QString getName() const { return "FPzip"; }
    QString getDescription() const { return FPR_DESCRIPTION; }
    QString getVersion() const {return FPR_VERSION; }
    QString getCopyright() const {

        QString copyright = FPR_COPYRIGHT;
        copyright += "\n Portions (C) Copyright Gilles Vollant (minizip)";
        copyright += "\n Portions (C) Copyright Jean-Ioup Gailly and Mark Adler (zlib)";

        return FPR_COPYRIGHT;
    }

    ImageFilePlugin::Features getFeatures() const {
        return ImageFilePlugin::F_Read
                | ImageFilePlugin::F_Write
                | ImageFilePlugin::F_FloppyDisk
                | ImageFilePlugin::F_Comment;
        // This plugin not support raw image files.
    }

private:

};

#endif // _FPZIP_H
