/*
-----------------------------------------------------
   Application Name: DIMG
          File Name: ziputil.h
         Start Date: 28/03/2010
       Last Changed: $LastChangedDate: 2011-04-09 00:17:11 +1200 (Sat, 09 Apr 2011) $
           Revision: $Revision: 295 $
          Author(s): David Goodwin
          Copyright: (C) Copyright David Goodwin, 2010
            License: GNU General Public License
   File Description: Various zip-related utility functions (header file)
-----------------------------------------------------
        This file is part of DIMG.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this software; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

File Notes:
------------------
*/


#ifndef ZIPUTIL_H
#define ZIPUTIL_H

// Qt headers
#include <QString>
#include <QIODevice>
#include <QDateTime>
#include <QProgressDialog>

/* Library Export Stuff */
/*
#ifdef __cplusplus
#define cppstuff "C"
#else
#define cppstuff
#endif

#ifdef BUILD_DLL
#define ZUEXPORT __declspec(dllexport)
#else
#define ZUEXPORT extern cppstuff __declspec(dllimport)
#endif */

#define ZUEXPORT

// Zip stuff
#define WRITEBUFFERSIZE (8192)
#define MAXFILENAME (256)
#define CASESENSITIVITY (0)

// From zlib 1.2.4
#define Z_NO_COMPRESSION         0
#define Z_BEST_SPEED             1
#define Z_BEST_COMPRESSION       9
#define Z_DEFAULT_COMPRESSION  (-1)

/** Adds a file to a zip archive creating the archive if it does not already exist.

  \param zipFileName The name of the zip archive to add the file to.
  \param fileData The file data to add to the zip archive.
  \param fileName The name to save the file data as in the zip archive.
  \param fileDateTime The date/time information to store with the file being added to the
         zip archive.
  \param fileComment The files comment.
  \param overwriteZipFileIfExists If the zip file exists it will be completely
         overwritten.
  \param appendZipFileIfExists If the specified zipFileName exists a new zip file will be
         appended on to the end of the file. This is only useful for making
         self-extractors, etc.
  \param compressFileData If the file data should be compressed or just archived
  \param compressionLevel the compression level - 0=fastest, 9=best.
  \param *qpd The progress dialog to use for the operation.
  \returns An error code.
 */
ZUEXPORT int writeFileToZip(QString zipFileName,
                            QIODevice *fileData,
                            QString fileName,
                            QDateTime fileDateTime,
                            QString fileComment,
                            bool overwriteZipFileIfExists,
                            bool appendZipFileIfExists,
                            bool compressFileData,
                            int compressionLevel,
                            QProgressDialog *qpd);

/** Sets the zip files comment. If the file does not already exist it will be
  created.
  \param zipFileName The name of the zip file that needs a new comment.
  \param comment The new comment.
  \returns An error code.
 */
ZUEXPORT int setZipFileComment(QString zipFileName, QString comment);

/** Gets the zip files comment.
  \param zipFileName the name of the zip file to get the comment for.
  \returns The comment or the empty string
 */
ZUEXPORT QString getZipFileComment(QString zipFileName);

/** Gets a list of all files in the specified zip file.
 \param zipFileName the zip file to get a listing for
 \param *fileList The QStringList to add file entries to.
 \returns An error code.
 */
ZUEXPORT int getZipFileList(QString zipFileName, QStringList *fileList);

/** Extracts the requested file from the specified zip archive and writes
  its data to the target parameter.
  \param zipFileName The zip file to open
  \param fileName The name of the file to extract from the archive.
  \param *target The QIODevice to write the files data to.
  \param *qpd The QProgressDialog to use during the operation.
  \returns An error code. 0 for success, 2 for file not found, anything else is an error
  of some form.
 */
ZUEXPORT int extractFileFromZipFile(QString zipFileName,
                                    QString fileName,
                                    QIODevice *target,
                                    QProgressDialog *qpd);

#endif // ZIPUTIL_H
