/*****************************************************************************
 *            Created: 7/04/2011
 *          Copyright: (C) Copyright David Goodwin, 2011
 *            License: GNU General Public License
 *        Description: FSEntry interface for FSPlugin v2
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#ifndef _FSENTRY2_H
#define _FSENTRY2_H

#include <fsentry.h>

class FSEntry2 : public FSEntry {

public:
    /** Returns the alternate base name of the file without the path or
      extension.

      The alternate base name contains all characters in the filename up to
      (but not including) the first '.' character.

      The alternate names are used by filesystems supporting more than one
      namespace (in FatFS v2 the alternate names allow access to the MS-DOS
      filename).

      \returns The base name of the entries alternate filename.
     */
    virtual QString altBaseName() const = 0;

    /** Returns the alternate filenames suffix. That is, every character after
      the first '.' character in the filename.

      The alternate names are used by filesystems supporting more than one
      namespace (in FatFS v2 the alternate names allow access to the MS-DOS
      filename).

      \returns The suffix of the entries alternate filename.
     */
    virtual QString altSuffix() const = 0;

    /** The full alternate FileName of the entry.

      The alternate names are used by filesystems supporting more than one
      namespace (in FatFS v2 the alternate names allow access to the MS-DOS
      filename).

      \returns The full alternate filename of the entry.
     */
    virtual QString altFileName() const = 0;

    /** Returns the alternate file ame, including the path.

      The alternate names are used by filesystems supporting more than one
      namespace (in FatFS v2 the alternate names allow access to the MS-DOS
      filename).

      \returns The alternate filename including the path.
     */
    virtual QString altFilePath() const = 0;
};

#endif // _FSENTRY2_H
