/*****************************************************************************
 *            Created: 31/03/2011
 *          Copyright: (C) Copyright David Goodwin, 2011
 *            License: GNU General Public License
 *        Description: FSPlugin v2 interface
 *****************************************************************************
 *
 *   This is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This software is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this software; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 ****************************************************************************/

#ifndef FSPLUGINV2_H
#define FSPLUGINV2_H

#include <FSPlugin.h>
#include <fsentryv2.h>

class FSPluginv2 : public FSPlugin {


public:
    /** Returns an FSEntry2 that represents the specified file or directory.

      The FSEntry2 returned by this function should represent the specified
      filename. If the specified file could not be found, it should just return
      null.

      It is the responsibility of the caller to delete the FSEntry2 returned
      by this method. The FSPlugin will not keep track of it.

      \param filename The name of the file to get the FSEntry2 for.
      \returns The FSEntry2 that represents the specified file or directory.
     */
    virtual FSEntry2 * getEntryByName(QString filename) = 0;


    /** Returns a list of all v2 FSEntries in the specified directory. If the
      directory could not be found, it should return an empty list.

      It is the responsibility of the caller to delete every FSEntry2 returned
      by this method. The FSPlugin will not keep track of them.

      \param directory The name of the directory to get an entry listing for.
      \returns An entry listing for the specified directory.
     */
    virtual QList<FSEntry2 *> getEntryList2(QString directory) = 0;

};

Q_DECLARE_INTERFACE(FSPluginv2, "nz.net.zx.sw.DIMG.FSPlugin/2.0");

#endif // FSPLUGINV2_H
